@@ -40,6 +40,7 @@ class PanelConfig(object):
     def __init__(self):
         self.desktops = []
         self.properties = {}
+        self.source = None
 
     def from_xfconf(xfconf):
         pc = PanelConfig()
@@ -73,14 +74,33 @@ class PanelConfig(object):
         config = pc.source.extractfile('config.txt')
 
         for line in config:
-            x = line.decode('utf-8').strip().split(' ', 1)
-            pc.properties[x[0]] = GLib.Variant.parse(None, x[1], None, None)
+            try:
+                x = line.decode('utf-8').strip().split(' ', 1)
+                pc.properties[x[0]] = GLib.Variant.parse(None, x[1], None, None)
+            except:
+                pass
 
         pc.find_desktops()
 
         return pc
 
+    def check_desktop(self, path):
+        bytes = self.get_desktop_source_file(path).read()
+
+        # Check if binary exists
+        keyfile = GLib.KeyFile.new()
+        decoded = bytes.decode()
+        if keyfile.load_from_data(decoded, len(decoded),
+                                  GLib.KeyFileFlags.NONE):
+            exec_str = keyfile.get_string("Desktop Entry", "Exec")
+            if self.check_exec(exec_str):
+                return True
+
+        return False
+
     def find_desktops(self):
+        remove_keys = []
+
         for pp, pv in self.properties.items():
             path = pp.split('/')
             if len(path) == 3 and path[0] == '' and path[1] == 'plugins' and \
@@ -90,10 +110,20 @@ class PanelConfig(object):
                         pv.get_string() == 'launcher':
                     for d in self.properties['/plugins/plugin-' + number +
                                              '/items'].unpack():
-                        self.desktops.append('launcher-' + number + '/' + d)
+                        desktop_path = 'launcher-' + number + '/' + d
+                        if self.check_desktop(desktop_path):
+                            self.desktops.append(desktop_path)
+                        else:
+                            remove_keys.append('/plugins/plugin-' + number)
+
+        keys = list(self.properties.keys())
+        for param in keys:
+            for bad_plugin in remove_keys:
+                if param.startswith(bad_plugin):
+                    self.properties.pop(param, None)
 
     def get_desktop_source_file(self, desktop):
-        if self.source is None:
+        if getattr(self, 'source', None) is None:
             path = os.path.join(
                 GLib.get_home_dir(), '.config/xfce4/panel/', desktop)
             return open(path, 'rb')
@@ -119,6 +149,23 @@ class PanelConfig(object):
 
         t.close()
 
+    def check_exec(self, program):
+        program = program.strip()
+        if len(program) == 0:
+            return False
+
+        params = list(GLib.shell_parse_argv(program)[1])
+        executable = params[0]
+
+        if os.path.exists(executable):
+            return True
+
+        path = GLib.find_program_in_path(executable)
+        if path is not None:
+            return True
+
+        return False
+
     def to_xfconf(self, xfconf):
         session_bus = Gio.BusType.SESSION
         conn = Gio.bus_get_sync(session_bus, None)
@@ -130,6 +177,13 @@ class PanelConfig(object):
         dbus_proxy = Gio.DBusProxy.new_sync(conn, 0, None, destination, path, interface, None)
 
         if dbus_proxy is not None:
+            # Reset all properties to make sure old settings are invalidated
+            try:
+                xfconf.call_sync('ResetProperty', GLib.Variant(
+                    '(ssb)', ('xfce4-panel', '/', True)), 0, -1, None)
+            except GLib.Error:
+                pass
+
             for (pp, pv) in sorted(self.properties.items()):
                 result = xfconf.call_sync('SetProperty', GLib.Variant(
                     '(ssv)', ('xfce4-panel', pp, pv)), 0, -1, None)
@@ -143,40 +197,7 @@ class PanelConfig(object):
                 f.write(bytes)
                 f.close()
 
-            dbus_proxy.call_sync('Terminate', GLib.Variant('(b)', ('xfce4-panel',)), 0, -1, None)
-
-
-if __name__ == '__main__':
-
-    import sys
-
-    session_bus = Gio.BusType.SESSION
-    cancellable = None
-    connection = Gio.bus_get_sync(session_bus, cancellable)
-
-    proxy_property = 0
-    interface_properties_array = None
-    destination = 'org.xfce.Xfconf'
-    path = '/org/xfce/Xfconf'
-    interface = destination
-
-    xfconf = Gio.DBusProxy.new_sync(
-        connection,
-        proxy_property,
-        interface_properties_array,
-        destination,
-        path,
-        interface,
-        cancellable)
-
-    if len(sys.argv) != 3 or sys.argv[1] not in ['load', 'save']:
-        print('Panel Switch v0.1 - Usage:')
-        print(sys.argv[0] + ' save <filename> : save current configuration.')
-        print(sys.argv[0] + ' load <filename> : load configuration from file.')
-        print('')
-        exit(-1)
-
-    if sys.argv[1] == 'save':
-        PanelConfig.from_xfconf(xfconf).to_file(sys.argv[2])
-    elif sys.argv[1] == 'load':
-        PanelConfig.from_file(sys.argv[2]).to_xfconf(xfconf)
+            try:
+                dbus_proxy.call_sync('Terminate', GLib.Variant('(b)', ('xfce4-panel',)), 0, -1, None)
+            except GLib.GError:
+                pass
